package com.ruoyi.demo.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.pagehelper.PageInfo;

import com.ruoyi.common.core.domain.R;
import com.ruoyi.common.core.validate.EditGroup;
import com.ruoyi.demo.entity.Brand;
import com.ruoyi.demo.entity.Enterprise;
import com.ruoyi.demo.entity.vo.BrandVo;
import com.ruoyi.demo.mapper.BrandMapper;
import com.ruoyi.demo.service.BrandRecycleService;
import com.ruoyi.demo.service.BrandService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotNull;
import java.util.HashMap;

@RestController
@RequestMapping("/recycle/brand")
public class BrandRecycleController {
    @Autowired
    BrandRecycleService brandRecycleService;

    @Autowired
    BrandService brandService;

    @GetMapping("/list")
    public R list(BrandVo brandVo){
        Page<Brand> list = brandRecycleService.list(brandVo);
        return R.ok(list);
    }

    /**
     * 移除品牌
     * @param brandId
     * @return
     */
    @DeleteMapping("/deleteBrand/{brandId}")
    public R deleteBrand(@PathVariable("brandId") String brandId){
        brandRecycleService.deleteBrand(brandId);
        return R.ok();
    }

    /**
     * 恢复品牌
     * @param hashMap
     * @return
     */
    @PutMapping("/recoverBrand")
    public R recoverBrand(@RequestBody HashMap<String,String> hashMap){
        if(hashMap.get("brandId") == null)
            return R.fail(400,"brandId不能为空");

        brandRecycleService.recoverBrand(hashMap.get("brandId"));
        return R.ok();
    }

    /**
     * 修改品牌
     * @param brandVo
     * @return
     */
    @PutMapping("/updateBrand")
    public R updateBrand(@Validated(value = {EditGroup.class}) @RequestBody BrandVo brandVo){
        brandService.updateBrand(brandVo);
        return R.ok();
    }

    /**
     * 简单品牌关联下拉框
     * @param brandName
     * @param size
     * @return
     */
    @GetMapping("/selectBrand")
    public R selectBrand(String brandName,Integer size){
        Page<Brand> brandPage = brandService.selectBrand(brandName, size);
        return R.ok(brandPage);
    }

    /**
     * 简单企业关联下拉框
     * @param enterpriseName
     * @param size
     * @return
     */
    @GetMapping("/selectEnterprise")
    public R selectEnterprise(String enterpriseName,Integer size){
        Page<Enterprise> enterprisePage = brandService.selectEnterprise(enterpriseName, size);
        return R.ok(enterprisePage);
    }

}
