package com.ruoyi.demo.controller;

import cn.dev33.satoken.annotation.SaIgnore;
import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.PageUtil;
import cn.hutool.json.JSONString;
import cn.hutool.json.JSONUtil;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.domain.R;
import com.ruoyi.common.helper.LoginHelper;
import com.ruoyi.common.utils.JsonUtils;
import com.ruoyi.demo.constant.RedisContant;
import com.ruoyi.demo.entity.WdInfo;
import com.ruoyi.demo.entity.bo.PointBo;
import com.ruoyi.demo.entity.vo.TopologicalWdAceeptVo;
import com.ruoyi.demo.service.TopologicalCommonService;
import com.ruoyi.demo.service.WdInfoService;
import io.swagger.v3.oas.models.security.SecurityScheme;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.geo.Point;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.servlet.mvc.method.annotation.SseEmitter;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;

/**
 * 点位评估-基本功能
 */
@RestController
@RequestMapping("/poi/common")
@Slf4j
public class TopologicalCommonController {
    @Autowired
    TopologicalCommonService topologicalCommonService;

    @Autowired
    WdInfoService wdInfoService;


    @Autowired
    RedisTemplate redisTemplate;

    /**
     * 搜索栏——前缀查询
     * @param searchText 搜索关键字
     * @param size 返回数量
     * @return
     */
    @RequestMapping("/prefixSearch")
    public R prefixSearch(String searchText,Integer size){
        List<WdInfo> wdInfos = topologicalCommonService.prefixSearch(searchText,size);
        return R.ok(wdInfos);
    }

    /**
     * 获取半径范围下拉框
     * @return
     */
    @RequestMapping("/range")
    public R range(){
        HashMap hashMap = new HashMap<>();
        hashMap.put("500m",500);
        hashMap.put("1km",1000);
        hashMap.put("2km",2000);
        return R.ok(hashMap);
    }

    /**
     * 获取中心点相邻点坐标（使用SSE连接）
     * @param topologicalWdAceeptVo 周边网点接受体
     * @return
     * @throws IOException
     */
    @SaIgnore
    @GetMapping(path = "/map",produces = {MediaType.TEXT_EVENT_STREAM_VALUE})
    public SseEmitter map(TopologicalWdAceeptVo topologicalWdAceeptVo) throws IOException {
        String md5 = topologicalWdAceeptVo.getHash();

        //2.查看redis中是否存在有缓存
        List<PointBo> wdCount = (List<PointBo>) redisTemplate.boundHashOps(RedisContant.TOPOLOGICAL_COMMON_MAP).get(md5);
        if (wdCount != null) {
            SseEmitter sseEmitter = getSseEmitter();
            sendFragment(sseEmitter,wdCount);
            return sseEmitter;
        }

        List<PointBo> map = topologicalCommonService.map(topologicalWdAceeptVo);
        redisTemplate.boundHashOps(RedisContant.TOPOLOGICAL_COMMON_MAP).put(md5,map);
        redisTemplate.expire(RedisContant.TOPOLOGICAL_COMMON_MAP,RedisContant.TOPOLOGICAL_COMMON_MAP_TIME, TimeUnit.MINUTES); //30分钟
        SseEmitter sseEmitter = getSseEmitter();
        sendFragment(sseEmitter,map);
        return sseEmitter;
    }

    /**
     * SSE连接：获取连接实体类SseEmitter
     * @return
     */
    public SseEmitter getSseEmitter(){
        SseEmitter sseEmitter = new SseEmitter();
        sseEmitter.onTimeout(() -> log.info("SSE推送超时"));
        sseEmitter.onCompletion(() -> log.info("SSE推送完成"));
        sseEmitter.onError((e) -> log.error("SSE推送异常",e));
        return sseEmitter;
    }

    /**
     * SSE连接：发送信息
     * @param sseEmitter
     * @param wdCount 点位信息
     */
    public void sendFragment(SseEmitter sseEmitter,List<PointBo> wdCount){
        int size = 500;
        int pages = PageUtil.totalPage(wdCount.size(), size);
        for (int i = 0; i < pages; i++) {
            try {
                List<PointBo> boList = CollUtil.page(i, size, wdCount);
                if (!boList.isEmpty()) sseEmitter.send(SseEmitter.event().name("message").data(JSONUtil.toJsonStr(boList)));
            } catch (IOException e) {
                sseEmitter.completeWithError(e);
            }
        }
        try {
            sseEmitter.send(SseEmitter.event().name("finish").data("数据推送完成"));
        } catch (IOException e) {
            sseEmitter.completeWithError(e);
        }
        sseEmitter.complete();
    }

}
