package com.ruoyi.demo.controller;

import com.ruoyi.common.core.domain.R;
import com.ruoyi.common.core.validate.EditGroup;
import com.ruoyi.demo.entity.*;
import com.ruoyi.demo.entity.vo.*;
import com.ruoyi.demo.mapper.WdInfoDao;
import com.ruoyi.demo.service.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;

/**
 * 精准营销管理中心——数据回收站
 */
@RestController
@RequestMapping("/recycle/wd")
public class WdRecycleController {


    @Autowired
    WdRecycleService wdRecycleService;

    @Autowired
    StoreWdService storeWdService;

    @Autowired
    HouseWdService houseWdService;

    @Autowired
    BuildWdService buildWdService;

    @Autowired
    TrafficWdService trafficWdService;

    @Autowired
    EnterpriseWdService enterpriseWdService;

    @Autowired
    WdInfoService wdInfoService;


    /**
     * 获取网点回收站列表
     * @param wdInfoVo 网点信息接受体
     * @return
     */
    @GetMapping("/list")
    public R list(WdInfoVo wdInfoVo){
        HashMap<String, Object> list = wdRecycleService.list(wdInfoVo);
        return R.ok(list);
    }


    /**
     * 移除网点
     * @param wdId 网点Id
     * @return
     */
    @DeleteMapping("/deleteWd/{wdId}")
    public R deleteWd(@PathVariable("wdId") String wdId){
        wdRecycleService.deleteWd(wdId);
        return R.ok();
    }

    /**
     * 恢复网点
     * @param wdInfoVo
     * @return
     */
    @PutMapping("/recoverWd")
    public R recoverWd(@RequestBody WdInfoVo wdInfoVo){
        if (wdInfoVo.getWdId() == null)
            return R.fail(400,"wdId不能为空");

        wdRecycleService.recoverWd(wdInfoVo.getWdId());
        return R.ok();
    }

    /**
     * 修改基础设施网点信息
     * @param wdInfoVo
     * @return
     */
    @PutMapping("/updateInfrastructureWd")
    public R updateWdInfo(@Validated(value = {EditGroup.class}) @RequestBody WdInfoVo wdInfoVo){
        wdInfoService.updateWdInfo(wdInfoVo);
        return R.ok();
    }

    /**
     * 修改门店网点信息
     * @param storeWdVo
     * @return
     */
    @PutMapping("/updateStoreWd")
    public R updateStoreWd(@Validated(value = {EditGroup.class}) @RequestBody StoreWdVo storeWdVo){
        storeWdService.updateStoreWd(storeWdVo);
        return R.ok();
    }


    /**
     * 修改小区网点信息
     * @param houseWdVo
     * @return
     */
    @PutMapping("/updateHouseWd")
    public R updateHouseWd(@Validated(value = {EditGroup.class}) @RequestBody HouseWdVo houseWdVo){
        houseWdService.updateHouseWd(houseWdVo);
        return R.ok();
    }

    /**
     * 修改楼宇网点信息
     * @param buildwdVo
     * @return
     */
    @PutMapping("/updateBuildWd")
    public R updateBuildWd(@Validated(value = {EditGroup.class}) @RequestBody BuildwdVo buildwdVo){
        buildWdService.updateBuildWd(buildwdVo);
        return R.ok();
    }

    /**
     * 修改交通设施网点信息
     * @param trafficWd
     * @return
     */
    @PutMapping("/updateTrafficWd")
    public R updateTrafficWd(@Validated(value = {EditGroup.class}) @RequestBody TrafficWdVo trafficWd){
        trafficWdService.updateTrafficWd(trafficWd);
        return R.ok();
    }

    /**
     * 修改企业网点信息
     * @param enterpriseWdVo
     * @return
     */
    @PutMapping("/updateEnterpriseWd")
    public R updateEnterpriseWd(@Validated(value = {EditGroup.class}) @RequestBody EnterpriseWdVo enterpriseWdVo){
        enterpriseWdService.updateEnterpriseWd(enterpriseWdVo);
        return R.ok();
    }
}
