package com.ruoyi.demo.service.impl;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ruoyi.demo.entity.AddrCategory;
import com.ruoyi.demo.entity.TypeBy;
import com.ruoyi.demo.entity.vo.AddrCategoryBody;
import com.ruoyi.demo.entity.vo.TypeByBody;
import com.ruoyi.demo.mapper.AddrCategoryDao;
import com.ruoyi.demo.mapper.TypeByDao;
import com.ruoyi.demo.service.CommonService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Service
@Transactional
public class CommonServiceImpl implements CommonService {

    @Autowired
    TypeByDao typeByDao;

    @Autowired
    AddrCategoryDao addrCategoryDao;

    /**
     * 获取犇云分类下拉框
     * @return
     */
    @Override
    public List<TypeByBody> tag() {
        //1.获取所有分类数据
        List<TypeBy> typeBIES = typeByDao.selectList(null);

        //2.找到所有根节点
        List<TypeBy> collect = typeBIES.stream().filter(typeBy -> {
            if (typeBy.getMidCategory() == null || typeBy.getMidCategory().equals(""))
                return true;
            return false;
        }).collect(Collectors.toList());

        //3.组装父子结构
        List<TypeByBody> typeByBodies = new ArrayList<>();
        for (TypeBy typeBy : collect) {
            TypeByBody typeByBody = new TypeByBody();
            typeByBody.setTypeCodeBy(typeBy.getTypeCodeBy());
            typeByBody.setTypeNameBy(typeBy.getBigCategory());
            typeByBody.setTypeByBodies(findChild(typeBIES, typeByBody.getTypeCodeBy(), 2));
            typeByBodies.add(typeByBody);
        }

        return typeByBodies;
    }

    /**
     * 分等级（省、市、区）查询地区
     * @param code 地区Code：level=province（不需要）、level=city（code=省码）、level=zone（code=市码）
     * @param level 搜索等级：province（查询全省）、city（查某个省的市）、zone（查某个市的区）
     * @return
     */
    @Override
    public List<AddrCategory> addrCodeLevel(String code,String level) {
        if ("province".equals(level)){
            QueryWrapper<AddrCategory> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("type","province");
            return addrCategoryDao.selectList(queryWrapper);
        }else if("city".equals(level)){
            QueryWrapper<AddrCategory> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("type","city");
            queryWrapper.and(categoryQueryWrapper -> {
                categoryQueryWrapper.likeRight("addr_code",code.substring(0,2));
            });
            return addrCategoryDao.selectList(queryWrapper);
        }else if("zone".equals(level)){
            QueryWrapper<AddrCategory> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("type","zone");
            queryWrapper.and(categoryQueryWrapper -> {
                categoryQueryWrapper.likeRight("addr_code",code.substring(0,4));
            });
            return addrCategoryDao.selectList(queryWrapper);
        }
        return null;
    }

    /**
     * 找到 大类 下的所有 子类
     * @param typeBIES 犇云分类数据集
     * @param typeCodeBy 大类Code
     * @param randType 子类等级（2是中类，4是小类）
     * @return
     */
    public List<TypeByBody> findChild(List<TypeBy> typeBIES, String typeCodeBy, int randType){
        String substring = typeCodeBy.substring(0, randType);
        if(randType == 2){
            List<TypeByBody> list = new ArrayList<>();
            for (TypeBy typeBY : typeBIES) {
                if(typeBY.getTypeCodeBy().startsWith(substring)  && (typeBY.getMidCategory() != null && !typeBY.getMidCategory().equals("")) && (typeBY.getSubCategory() == null || typeBY.getSubCategory().equals(""))){
                    TypeByBody typeByBody = new TypeByBody();
                    typeByBody.setTypeCodeBy(typeBY.getTypeCodeBy());
                    typeByBody.setTypeNameBy(typeBY.getMidCategory());
                    typeByBody.setTypeByBodies(findChild(typeBIES, typeByBody.getTypeCodeBy(), 4));

                    list.add(typeByBody);
                }
            }
            return list;
        }else if(randType == 4){
            List<TypeByBody> list = new ArrayList<>();
            for (TypeBy typeBY : typeBIES) {
                if(typeBY.getTypeCodeBy().startsWith(substring)  && typeBY.getSubCategory() != null && !typeBY.getSubCategory().equals("")){
                    TypeByBody typeByBody = new TypeByBody();
                    typeByBody.setTypeCodeBy(typeBY.getTypeCodeBy());
                    typeByBody.setTypeNameBy(typeBY.getSubCategory());
                    typeByBody.setTypeByBodies(null);

                    list.add(typeByBody);
                }
            }
            return list;
        }
        return null;
    }

    /**
     * 获取地区addrCodeMap,并组成父子结构
     * @return
     */
    @Override
    public List<AddrCategoryBody> addrCodeMap() {
        //1.获取所有地区Code
        List<AddrCategory> addrCategories = addrCategoryDao.selectList(null);

        //2.找到所有根节点
        List<AddrCategory> collect = addrCategories.stream().filter(addrCategory -> {
            if (addrCategory.getCity() == null || addrCategory.getCity().equals(""))
                return true;
            return false;
        }).collect(Collectors.toList());

        //3.组装父子结构
        List<AddrCategoryBody> addrCategoryBodies = new ArrayList<>();
        for (AddrCategory addrCategory : collect) {
            AddrCategoryBody addrCategoryBody = new AddrCategoryBody();
            addrCategoryBody.setAddrCoreName(addrCategory.getProvince());
            addrCategoryBody.setAddrCore(addrCategory.getAddrCode());
            addrCategoryBody.setAddrCategoryes(findChildAddr(addrCategories, addrCategoryBody.getAddrCore(), 2));

            addrCategoryBodies.add(addrCategoryBody);
        }

        return addrCategoryBodies;
    }

    /**
     * 找到 大类 下的所有 子类
     * @param addrCategories 地区数据集
     * @param addrCore 大类Code
     * @param randType 子类等级（2是中类，4是小类）
     * @return
     */
    public List<AddrCategoryBody> findChildAddr(List<AddrCategory> addrCategories, String addrCore, int randType){
        String substring = addrCore.substring(0, randType);
        if(randType == 2){
            List<AddrCategoryBody> list = new ArrayList<>();
            for (AddrCategory addrCategory : addrCategories) {
                if(addrCategory.getAddrCode().startsWith(substring)  && (addrCategory.getCity() != null && !addrCategory.getCity().equals("")) && (addrCategory.getDistrict() == null || addrCategory.getDistrict().equals(""))){
                    AddrCategoryBody addrCategoryBody = new AddrCategoryBody();
                    addrCategoryBody.setAddrCoreName(addrCategory.getCity());
                    addrCategoryBody.setAddrCore(addrCategory.getAddrCode());
                    addrCategoryBody.setAddrCategoryes(findChildAddr(addrCategories, addrCategoryBody.getAddrCore(), 4));

                    list.add(addrCategoryBody);
                }
            }
            return list;
        }else if(randType == 4){
            List<AddrCategoryBody> list = new ArrayList<>();
            for (AddrCategory addrCategory : addrCategories) {
                if(addrCategory.getAddrCode().startsWith(substring)  && addrCategory.getDistrict() != null && !addrCategory.getDistrict().equals("")){
                    AddrCategoryBody addrCategoryBody = new AddrCategoryBody();
                    addrCategoryBody.setAddrCoreName(addrCategory.getDistrict());
                    addrCategoryBody.setAddrCore(addrCategory.getAddrCode());
                    addrCategoryBody.setAddrCategoryes(null);

                    list.add(addrCategoryBody);
                }
            }
            return list;
        }
        return null;
    }
}
