package com.ruoyi.demo.service.impl;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.demo.entity.WdInfo;
import com.ruoyi.demo.entity.bo.ChannelBo;
import com.ruoyi.demo.entity.vo.AddStatusVo;
import com.ruoyi.demo.entity.vo.ChannelVo;
import com.ruoyi.demo.entity.vo.TimeVo;
import com.ruoyi.demo.entity.vo.WdInfoVo;
import com.ruoyi.demo.mapper.StoreWdOpcloseDao;
import com.ruoyi.demo.mapper.WdInfoMapper;
import com.ruoyi.demo.mapper.WdOpcloseDao;
import com.ruoyi.demo.service.WdInfoService;
import com.ruoyi.demo.utils.CategoryUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service
@Transactional
public class WdInfoServiceImpl implements WdInfoService {
    @Autowired
    CategoryUtil categoryUtil;
    @Autowired
    WdInfoMapper wdInfoMapper;
    @Autowired
    WdOpcloseDao wdOpcloseDao;
    @Autowired
    StoreWdOpcloseDao storeWdOpcloseDao;

    @Override
    public List<AddStatusVo> searchAddStatus(List<String> addrCodes, List<String> typeCodes) {
        // 初始化时间信息，补齐地址信息
        categoryUtil.setWdTypeMap();
        List<String> addrs = new ArrayList<>();
        List<String> times = new ArrayList<>();
        for (int i = 0; i < 12; i++)
            // times.add(LocalDateTime.now(ZoneId.of("Asia/Shanghai")).plusMonths(-i).format(DateTimeFormatter.ofPattern("yyyy-MM")));
            times.add(DateUtil.offsetMonth(new Date(), -i).toString("yyyy-MM"));
        if (addrCodes != null) {
            if (!addrCodes.isEmpty())
                for (String s : addrCodes)
                    addrs.addAll(categoryUtil.getAllOtherAddrCode(s));
        } else
            addrs = null;

        List<AddStatusVo> addStatusVos = wdInfoMapper.searchAddStatusByMulti(times, addrs, typeCodes);
        addStatusVos.forEach(vo -> vo.setTypeName(categoryUtil.wdTypeMap.get(vo.getTypeCode())));
        return addStatusVos;
    }

    public Integer timeUtil(String time) {
        String[] split = time.split(":");
        int hour = Integer.parseInt(split[0]);
        int min = Integer.parseInt(split[1]);
        return hour * 100 + min;
    }

    public int compareTime(String time1, String time2) {
        return timeUtil(time1).compareTo(timeUtil(time2));
    }

    @Override
    public List<TimeVo> searchOpenTimes(List<String> addrCodes, List<String> typeCodeBys) {
        // 补齐地址码和类型码
        List<String> addrs = new ArrayList<>();
        if (addrCodes != null) {
            if (!addrCodes.isEmpty())
                for (String addr : addrCodes)
                    addrs.addAll(categoryUtil.getAllOtherAddrCode(addr));
        } else
            addrs = null;
        List<String> types = new ArrayList<>();
        if (typeCodeBys != null) {
            if (!typeCodeBys.isEmpty())
                for (String type : typeCodeBys)
                    types.addAll(categoryUtil.getAllOtherTypeBy(type));
        } else
            types = null;

        List<TimeVo> vos = storeWdOpcloseDao.selectWdOpenTimes(addrs, types);
        // 初始化
        List<TimeVo> rvos = new ArrayList<>();
        for (int i = 0; i < 24; i += 2) {
            TimeVo vo = new TimeVo();
            vo.setTime(i + ":00~" + (i + 2) + ":00");
            vo.setCount(0);
            rvos.add(vo);
        }
        // 赋值
        vos.forEach(vo -> {
            for (TimeVo rvo : rvos) {
                String[] times = rvo.getTime().split("~");
                if (compareTime(vo.getTime(), times[0]) < 0 || compareTime(times[1], vo.getTime()) < 0)
                    continue;
                rvo.setCount(rvo.getCount() + vo.getCount());
            }
        });
        return rvos;
    }

    @Override
    public List<TimeVo> searchCloseTimes(List<String> addrCodes, List<String> typeCodeBys) {
        // 补齐地址码和类型码
        List<String> addrs = new ArrayList<>();
        if (addrCodes != null) {
            if (!addrCodes.isEmpty())
                for (String addr : addrCodes)
                    addrs.addAll(categoryUtil.getAllOtherAddrCode(addr));
        } else
            addrs = null;
        List<String> types = new ArrayList<>();
        if (typeCodeBys != null) {
            if (!typeCodeBys.isEmpty())
                for (String type : typeCodeBys)
                    types.addAll(categoryUtil.getAllOtherTypeBy(type));
        } else
            types = null;

        List<TimeVo> vos = storeWdOpcloseDao.selectWdCloseTimes(addrs, types);
        // 初始化
        List<TimeVo> rvos = new ArrayList<>();
        for (int i = 0; i < 24; i += 2) {
            TimeVo vo = new TimeVo();
            vo.setTime(i + ":00~" + (i + 2) + ":00");
            vo.setCount(0);
            rvos.add(vo);
        }
        // 赋值
        vos.forEach(vo -> {
            for (TimeVo rvo : rvos) {
                String[] times = rvo.getTime().split("~");
                if (compareTime(vo.getTime(), times[0]) < 0 || compareTime(times[1], vo.getTime()) < 0)
                    continue;
                rvo.setCount(rvo.getCount() + vo.getCount());
            }
        });
        return rvos;
    }

    @Override
    public Page<ChannelVo> searchChannel(ChannelBo bo) {
        QueryWrapper<WdInfo> wdInfoQueryWrapper = new QueryWrapper<>();
        wdInfoQueryWrapper.ne("dsw.channel_type", "无");
        if (bo.getText() != null) {
            wdInfoQueryWrapper.and(wrapper -> {
                wrapper.like("dwi.wd_name", bo.getText())
                    .or()
                    .like("dsw.contact", bo.getText())
                    .or()
                    .like("dsw.telephone", bo.getText());
            });
        }
        // 网点分类（犇云）
        List<String> types = new ArrayList<>();
        if (bo.getTypeCodeBys() != null)
            if (bo.getTypeCodeBys().size() > 0) {
                for (String type : bo.getTypeCodeBys())
                    types.addAll(categoryUtil.getAllOtherTypeBy(type));
            } else
                types = null;
        else types = null;
        if (types != null)
            wdInfoQueryWrapper.in("dwi.type_code_by", types);

        // 地区码
        List<String> addrs = new ArrayList<>();
        if (bo.getAddrCodes() != null)
            if (bo.getAddrCodes().size() > 0) {
                for (String addr : bo.getAddrCodes())
                    addrs.addAll(categoryUtil.getAllOtherAddrCode(addr));
            } else
                addrs = null;
        else addrs = null;
        if (addrs != null)
            wdInfoQueryWrapper.in("dwi.addr_code", addrs);

        // 潜力值
        if (bo.getIndexScore() != null)
            if (bo.getIndexScore().size() == 2)
                wdInfoQueryWrapper.ge("index_score", bo.getIndexScore().get(0))
                    .le("index_score", bo.getIndexScore().get(1));

        Page<WdInfo> page = new Page<>(bo.getPageNum(), bo.getPageSize());
        return wdInfoMapper.selectChannel(page, wdInfoQueryWrapper);
    }

    /**
     * 修改网点基本信息
     *
     * @param wdInfoVo
     */
    @Override
    public void updateWdInfo(WdInfoVo wdInfoVo) {
        //1.修改网点基本信息
        UpdateWrapper<WdInfo> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("wd_id", wdInfoVo.getWdId())
            .set("wd_name", wdInfoVo.getWdName())
            .set("addr_info", wdInfoVo.getAddrInfo())
            .set("addr_code", wdInfoVo.getAddrCode())
            .set("lat", wdInfoVo.getLat())
            .set("lng", wdInfoVo.getLng())
            .set("type_code_by", wdInfoVo.getTypeCodeBy())
            .set("type_name_by", wdInfoVo.getTypeNameBy())
            .set("update_time", DateUtils.getTime());
        wdInfoMapper.update(null, updateWrapper);

    }
}
